import pytest
import sys
import os
import importlib
import logging
import time
import yaml
from .utils import common as utl
from .server import OpenApiServer
from .grpcserver import grpc_server, GRPC_PORT
from .server import app


# TBD: fix this hardcoding
# artifacts should not be generated from here as these tests are run as sudo
pytest.module_name = "sanity"
pytest.artifacts_path = os.path.join(
    os.path.dirname(__file__), "..", "..", "art"
)
sys.path.append(pytest.artifacts_path)
sys.path.append(
    os.path.join(
        os.path.dirname(__file__), "..", "..", "art", pytest.module_name
    )
)

pytest.module = importlib.import_module(pytest.module_name)
pytest.http_server = OpenApiServer(pytest.module).start()
pytest.pb2_module = importlib.import_module(pytest.module_name + "_pb2")
pytest.pb2_grpc_module = importlib.import_module(
    pytest.module_name + "_pb2_grpc"
)
pytest.grpc_server = grpc_server()


@pytest.fixture(scope="session")
def api():
    """Return an instance of the top level Api class from the generated package"""
    module = importlib.import_module(pytest.module_name)
    api = module.api(
        location="http://127.0.0.1:{}".format(app.PORT),
        verify=False,
        logger=None,
        loglevel=logging.DEBUG,
    )
    # verify http server is up
    attempts = 1
    while True:
        try:
            api.get_config()
            break
        except Exception as e:
            print(e)
            if attempts > 5:
                raise (e)
        time.sleep(0.5)
        attempts += 1
    return api


@pytest.fixture(scope="session")
def grpc_api():
    """Return an instance of the top level gRPC Api class from the generated package"""
    return pytest.module.api(
        location="localhost:{}".format(GRPC_PORT),
        transport=pytest.module.Transport.GRPC,
        logger=None,
        loglevel=logging.DEBUG,
    )


@pytest.fixture(scope="session")
def proto_file_name():
    art_dir = os.path.join(os.path.dirname(__file__), "..", "..", "art")
    proto_file = os.path.join(art_dir, "{}.proto".format(pytest.module_name))
    return proto_file


@pytest.fixture
def config(api):
    """Return a new instance of an empty config"""
    config = api.prefix_config()
    config.a = "asdf"
    config.b = 1.1
    config.c = 1
    config.required_object.e_a = 1.1
    config.required_object.e_b = 1.2
    return config


@pytest.fixture
def default_config(api):
    config = api.prefix_config()
    config.a = "asdf"
    config.b = 1.1
    config.c = 1
    config.required_object.e_a = 1.1
    config.required_object.e_b = 1.2
    return config


@pytest.fixture
def pb_config(pb2):
    """Returns an instance of config of *_pb2 package"""
    return pb2.PrefixConfig()


@pytest.fixture
def utils():
    return utl


@pytest.fixture(scope="session")
def pb2():
    """Returns pb2 package"""
    return pytest.pb2_module


@pytest.fixture(scope="session")
def pb2_grpc():
    """Returns pb2_grpc package"""
    return pytest.pb2_grpc_module


@pytest.fixture
def openapi_yaml():
    path = os.path.join(pytest.artifacts_path, "openapi.yaml")
    _openapi = None
    with open(path) as fp:
        _openapi = yaml.safe_load(fp.read())
    return _openapi



import os
import pytest
from openapiart.openapiart import OpenApiArt as openapiart_class


def create_openapi_artifacts(openapiart_class, sdk=None, file_name=None):
    open_api = openapiart_class(
        api_files=[
            os.path.join(os.path.dirname(__file__), "./api/info.yaml"),
            os.path.join(os.path.dirname(__file__), file_name),
        ],
        artifact_dir=os.path.join(
            os.path.dirname(__file__), "..", "..", "fielduid"
        ),
        extension_prefix="field",
        proto_service="fieldapi",
    )
    if sdk == "python" or sdk is None:
        open_api.GeneratePythonSdk(package_name="fieldapi")


def str_compare(validte_str, entire_str):
    return validte_str in entire_str


def test_validate_property_name_camel_case():
    include_error = "is invalid. Only lower case letters separated with an underscore is allowed"
    with pytest.raises(Exception) as execinfo:
        create_openapi_artifacts(
            openapiart_class,
            file_name="./field_uid/property_name_camel_case.yaml",
        )
    error_value = execinfo.value.args[0]
    assert str_compare(include_error, error_value)


def test_validate_property_name_pascal_case():
    include_error = "is invalid. Only lower case letters separated with an underscore is allowed"
    with pytest.raises(Exception) as execinfo:
        create_openapi_artifacts(
            openapiart_class,
            file_name="./field_uid/property_name_pascal_case.yaml",
        )
    error_value = execinfo.value.args[0]
    assert str_compare(include_error, error_value)


def test_validate_property_name_upper_case():
    include_error = "is invalid. Only lower case letters separated with an underscore is allowed"
    with pytest.raises(Exception) as execinfo:
        create_openapi_artifacts(
            openapiart_class,
            file_name="./field_uid/property_name_upper_case.yaml",
        )
    error_value = execinfo.value.args[0]
    assert str_compare(include_error, error_value)


if __name__ == "__main__":
    pytest.main(["-v", "-s", __file__])



import importlib
import logging
import json
import platform
import yaml
import requests
import urllib3
import io
import sys
import time
import grpc
import semantic_version
import types
import platform
from google.protobuf import json_format
import sanity_pb2_grpc as pb2_grpc
import sanity_pb2 as pb2

try:
    from typing import Union, Dict, List, Any, Literal
except ImportError:
    from typing_extensions import Literal


if sys.version_info[0] == 3:
    unicode = str


openapi_warnings = []


class Transport:
    HTTP = "http"
    GRPC = "grpc"


def api(
    location=None,
    transport=None,
    verify=True,
    logger=None,
    loglevel=logging.INFO,
    ext=None,
    version_check=False,
):
    """Create an instance of an Api class

    generator.Generator outputs a base Api class with the following:
    - an abstract method for each OpenAPI path item object
    - a concrete properties for each unique OpenAPI path item parameter.

    generator.Generator also outputs an HttpApi class that inherits the base
    Api class, implements the abstract methods and uses the common HttpTransport
    class send_recv method to communicate with a REST based server.

    Args
    ----
    - location (str): The location of an Open Traffic Generator server.
    - transport (enum["http", "grpc"]): Transport Type
    - verify (bool): Verify the server's TLS certificate, or a string, in which
      case it must be a path to a CA bundle to use. Defaults to `True`.
      When set to `False`, requests will accept any TLS certificate presented by
      the server, and will ignore hostname mismatches and/or expired
      certificates, which will make your application vulnerable to
      man-in-the-middle (MitM) attacks. Setting verify to `False`
      may be useful during local development or testing.
    - logger (logging.Logger): A user defined logging.logger, if none is provided
      then a default logger with a stdout handler will be provided
    - loglevel (logging.loglevel): The logging package log level.
      The default loglevel is logging.INFO
    - ext (str): Name of an extension package
    """
    params = locals()
    transport_types = ["http", "grpc"]
    if ext is None:
        transport = "http" if transport is None else transport
        if transport not in transport_types:
            raise Exception(
                "{transport} is not within valid transport types {transport_types}".format(
                    transport=transport, transport_types=transport_types
                )
            )
        if transport == "http":
            return HttpApi(**params)
        else:
            return GrpcApi(**params)
    try:
        if transport is not None:
            raise Exception(
                "ext and transport are not mutually exclusive. Please configure one of them."
            )
        lib = importlib.import_module("{}_{}".format(__name__, ext))
        return lib.Api(**params)
    except ImportError as err:
        msg = "Extension %s is not installed or invalid: %s"
        raise Exception(msg % (ext, err))


class HttpTransport(object):
    def __init__(self, **kwargs):
        """Use args from api() method to instantiate an HTTP transport"""
        self.location = (
            kwargs["location"]
            if "location" in kwargs and kwargs["location"] is not None
            else "https://localhost:443"
        )
        self.verify = kwargs["verify"] if "verify" in kwargs else False
        self.logger = kwargs["logger"] if "logger" in kwargs else None
        self.loglevel = (
            kwargs["loglevel"] if "loglevel" in kwargs else logging.DEBUG
        )
        if self.logger is None:
            stdout_handler = logging.StreamHandler(sys.stdout)
            formatter = logging.Formatter(
                fmt="%(asctime)s [%(name)s] [%(levelname)s] %(message)s",
                datefmt="%Y-%m-%d %H:%M:%S",
            )
            formatter.converter = time.gmtime
            stdout_handler.setFormatter(formatter)
            self.logger = logging.Logger(self.__module__, level=self.loglevel)
            self.logger.addHandler(stdout_handler)
        self.logger.debug(
            "HttpTransport args: {}".format(
                ", ".join(["{}={!r}".format(k, v) for k, v in kwargs.items()])
            )
        )
        self.set_verify(self.verify)
        self._session = requests.Session()

    def set_verify(self, verify):
        self.verify = verify
        if self.verify is False:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
            self.logger.warning("Certificate verification is disabled")

    def send_recv(
        self,
        method,
        relative_url,
        payload=None,
        return_object=None,
        headers=None,
    ):
        url = "%s%s" % (self.location, relative_url)
        data = None
        headers = headers or {"Content-Type": "application/json"}
        if payload is not None:
            if isinstance(payload, bytes):
                data = payload
                headers["Content-Type"] = "application/octet-stream"
            elif isinstance(payload, (str, unicode)):
                data = payload
            elif isinstance(payload, OpenApiBase):
                data = payload.serialize()
            else:
                raise Exception("Type of payload provided is unknown")
        response = self._session.request(
            method=method,
            url=url,
            data=data,
            verify=False,
            allow_redirects=True,
            # TODO: add a timeout here
            headers=headers,
        )
        if response.ok:
            if "application/json" in response.headers["content-type"]:
                # TODO: we might want to check for utf-8 charset and decode
                # accordingly, but current impl works for now
                response_dict = yaml.safe_load(response.text)
                if return_object is None:
                    # if response type is not provided, return dictionary
                    # instead of python object
                    return response_dict
                else:
                    return return_object.deserialize(response_dict)
            elif (
                "application/octet-stream" in response.headers["content-type"]
            ):
                return io.BytesIO(response.content)
            else:
                # TODO: for now, return bare response object for unknown
                # content types
                return response
        else:
            raise Exception(
                response.status_code, yaml.safe_load(response.text)
            )


class OpenApiStatus:
    messages = {}
    # logger = logging.getLogger(__module__)

    @classmethod
    def warn(cls, key, object):
        if cls.messages.get(key) is not None:
            if cls.messages[key] in object.__warnings__:
                return
            # cls.logger.warning(cls.messages[key])
            logging.warning(cls.messages[key])
            object.__warnings__.append(cls.messages[key])
            # openapi_warnings.append(cls.messages[key])

    @staticmethod
    def deprecated(func_or_data):
        def inner(self, *args, **kwargs):
            OpenApiStatus.warn(
                "{}.{}".format(type(self).__name__, func_or_data.__name__),
                self,
            )
            return func_or_data(self, *args, **kwargs)

        if isinstance(func_or_data, types.FunctionType):
            return inner
        OpenApiStatus.warn(func_or_data)

    @staticmethod
    def under_review(func_or_data):
        def inner(self, *args, **kwargs):
            OpenApiStatus.warn(
                "{}.{}".format(type(self).__name__, func_or_data.__name__),
                self,
            )
            return func_or_data(self, *args, **kwargs)

        if isinstance(func_or_data, types.FunctionType):
            return inner
        OpenApiStatus.warn(func_or_data)


class OpenApiBase(object):
    """Base class for all generated classes"""

    JSON = "json"
    YAML = "yaml"
    DICT = "dict"

    __slots__ = ()

    __constraints__ = {"global": []}
    __validate_latter__ = {"unique": [], "constraint": []}

    def __init__(self):
        pass

    def serialize(self, encoding=JSON):
        """Serialize the current object according to a specified encoding.

        Args
        ----
        - encoding (str[json, yaml, dict]): The object will be recursively
            serialized according to the specified encoding.
            The supported encodings are json, yaml and python dict.

        Returns
        -------
        - obj(Union[str, dict]): A str or dict object depending on the specified
            encoding. The json and yaml encodings will return a str object and
            the dict encoding will return a python dict object.
        """
        # TODO: restore behavior
        # self._clear_globals()
        if encoding == OpenApiBase.JSON:
            data = json.dumps(self._encode(), indent=2, sort_keys=True)
        elif encoding == OpenApiBase.YAML:
            data = yaml.safe_dump(self._encode())
        elif encoding == OpenApiBase.DICT:
            data = self._encode()
        else:
            raise NotImplementedError("Encoding %s not supported" % encoding)
        # TODO: restore behavior
        # self._validate_coded()
        return data

    def _encode(self):
        raise NotImplementedError()

    def deserialize(self, serialized_object):
        """Deserialize a python object into the current object.

        If the input `serialized_object` does not match the current
        openapi object an exception will be raised.

        Args
        ----
        - serialized_object (Union[str, dict]): The object to deserialize.
            If the serialized_object is of type str then the internal encoding
            of the serialized_object must be json or yaml.

        Returns
        -------
        - obj(OpenApiObject): This object with all the
            serialized_object deserialized within.
        """
        # TODO: restore behavior
        # self._clear_globals()
        if isinstance(serialized_object, (str, unicode)):
            serialized_object = yaml.safe_load(serialized_object)
        self._decode(serialized_object)
        # TODO: restore behavior
        # self._validate_coded()
        return self

    def _decode(self, dict_object):
        raise NotImplementedError()

    def warnings(self):
        warns = list(self.__warnings__)
        if "2.7" in platform.python_version().rsplit(".", 1)[0]:
            del self.__warnings__[:]
        else:
            self.__warnings__.clear()
        return warns


class OpenApiValidator(object):

    __slots__ = ()

    _validation_errors = []

    def __init__(self):
        pass

    def _clear_errors(self):
        if "2.7" in platform.python_version().rsplit(".", 1)[0]:
            del self._validation_errors[:]
        else:
            self._validation_errors.clear()

    def validate_mac(self, mac):
        if (
            mac is None
            or not isinstance(mac, (str, unicode))
            or mac.count(" ") != 0
        ):
            return False
        try:
            if len(mac) != 17:
                return False
            return all([0 <= int(oct, 16) <= 255 for oct in mac.split(":")])
        except Exception:
            return False

    def validate_ipv4(self, ip):
        if (
            ip is None
            or not isinstance(ip, (str, unicode))
            or ip.count(" ") != 0
        ):
            return False
        if len(ip.split(".")) != 4:
            return False
        try:
            return all([0 <= int(oct) <= 255 for oct in ip.split(".", 3)])
        except Exception:
            return False

    def validate_ipv6(self, ip):
        if ip is None or not isinstance(ip, (str, unicode)):
            return False
        ip = ip.strip()
        if (
            ip.count(" ") > 0
            or ip.count(":") > 7
            or ip.count("::") > 1
            or ip.count(":::") > 0
        ):
            return False
        if (ip[0] == ":" and ip[:2] != "::") or (
            ip[-1] == ":" and ip[-2:] != "::"
        ):
            return False
        if ip.count("::") == 0 and ip.count(":") != 7:
            return False
        if ip == "::":
            return True
        if ip[:2] == "::":
            ip = ip.replace("::", "0:")
        elif ip[-2:] == "::":
            ip = ip.replace("::", ":0")
        else:
            ip = ip.replace("::", ":0:")
        try:
            return all(
                [
                    True
                    if (0 <= int(oct, 16) <= 65535) and (1 <= len(oct) <= 4)
                    else False
                    for oct in ip.split(":")
                ]
            )
        except Exception:
            return False

    def validate_hex(self, hex):
        if hex is None or not isinstance(hex, (str, unicode)):
            return False
        try:
            int(hex, 16)
            return True
        except Exception:
            return False

    def validate_integer(self, value, min, max, type_format=None):
        if value is None or not isinstance(value, int):
            return False
        if type_format == "uint32" and value < 0:
            return False
        if type_format == "uint64" and value < 0:
            return False
        if min is not None and value < min:
            return False
        if max is not None and value > max:
            return False
        return True

    def validate_float(self, value):
        return isinstance(value, (int, float))

    def validate_string(self, value, min_length, max_length):
        if value is None or not isinstance(value, (str, unicode)):
            return False
        if min_length is not None and len(value) < min_length:
            return False
        if max_length is not None and len(value) > max_length:
            return False
        return True

    def validate_bool(self, value):
        return isinstance(value, bool)

    def validate_list(self, value, itemtype, min, max, min_length, max_length):
        if value is None or not isinstance(value, list):
            return False
        v_obj = getattr(self, "validate_{}".format(itemtype), None)
        if v_obj is None:
            raise AttributeError(
                "{} is not a valid attribute".format(itemtype)
            )
        v_obj_lst = []
        for item in value:
            if itemtype == "integer":
                v_obj_lst.append(v_obj(item, min, max))
            elif itemtype == "string":
                v_obj_lst.append(v_obj(item, min_length, max_length))
            else:
                v_obj_lst.append(v_obj(item))
        return v_obj_lst

    def validate_binary(self, value):
        if value is None or not isinstance(value, (str, unicode)):
            return False
        return all(
            [
                True if int(bin) == 0 or int(bin) == 1 else False
                for bin in value
            ]
        )

    def types_validation(
        self,
        value,
        type_,
        err_msg,
        itemtype=None,
        min=None,
        max=None,
        min_length=None,
        max_length=None,
    ):
        type_map = {
            int: "integer",
            str: "string",
            float: "float",
            bool: "bool",
            list: "list",
            "int64": "integer",
            "int32": "integer",
            "uint64": "integer",
            "uint32": "integer",
            "double": "float",
        }
        type_format = type_
        if type_ in type_map:
            type_ = type_map[type_]
        if itemtype is not None and itemtype in type_map:
            itemtype = type_map[itemtype]
        v_obj = getattr(self, "validate_{}".format(type_), None)
        if v_obj is None:
            msg = "{} is not a valid or unsupported format".format(type_)
            raise TypeError(msg)
        if type_ == "list":
            verdict = v_obj(value, itemtype, min, max, min_length, max_length)
            if all(verdict) is True:
                return
            err_msg = "{} \n {} are not valid".format(
                err_msg,
                [
                    value[index]
                    for index, item in enumerate(verdict)
                    if item is False
                ],
            )
            verdict = False
        elif type_ == "integer":
            verdict = v_obj(value, min, max, type_format)
            if verdict is True:
                return
            min_max = ""
            if min is not None:
                min_max = ", expected min {}".format(min)
            if max is not None:
                min_max = min_max + ", expected max {}".format(max)
            err_msg = "{} \n got {} of type {} {}".format(
                err_msg, value, type(value), min_max
            )
        elif type_ == "string":
            verdict = v_obj(value, min_length, max_length)
            if verdict is True:
                return
            msg = ""
            if min_length is not None:
                msg = ", expected min {}".format(min_length)
            if max_length is not None:
                msg = msg + ", expected max {}".format(max_length)
            err_msg = "{} \n got {} of type {} {}".format(
                err_msg, value, type(value), msg
            )
        else:
            verdict = v_obj(value)
        if verdict is False:
            raise TypeError(err_msg)

    def _validate_unique_and_name(self, name, value, latter=False):
        if self._TYPES[name].get("unique") is None or value is None:
            return
        if latter is True:
            self.__validate_latter__["unique"].append(
                (self._validate_unique_and_name, name, value)
            )
            return
        class_name = type(self).__name__
        unique_type = self._TYPES[name]["unique"]
        if class_name not in self.__constraints__:
            self.__constraints__[class_name] = dict()
        if unique_type == "global":
            values = self.__constraints__["global"]
        else:
            values = self.__constraints__[class_name]
        if value in values:
            self._validation_errors.append(
                "{} with {} already exists".format(name, value)
            )
            return
        if isinstance(values, list):
            values.append(value)
        self.__constraints__[class_name].update({value: self})

    def _validate_constraint(self, name, value, latter=False):
        cons = self._TYPES[name].get("constraint")
        if cons is None or value is None:
            return
        if latter is True:
            self.__validate_latter__["constraint"].append(
                (self._validate_constraint, name, value)
            )
            return
        found = False
        for c in cons:
            klass, prop = c.split(".")
            names = self.__constraints__.get(klass, {})
            props = [obj._properties.get(prop) for obj in names.values()]
            if value in props:
                found = True
                break
        if found is not True:
            self._validation_errors.append(
                "{} is not a valid type of {}".format(value, "||".join(cons))
            )
            return

    def _validate_coded(self):
        for item in self.__validate_latter__["unique"]:
            item[0](item[1], item[2])
        for item in self.__validate_latter__["constraint"]:
            item[0](item[1], item[2])
        self._clear_vars()
        if len(self._validation_errors) > 0:
            errors = "\n".join(self._validation_errors)
            self._clear_errors()
            raise Exception(errors)

    def _clear_vars(self):
        if platform.python_version_tuple()[0] == "2":
            self.__validate_latter__["unique"] = []
            self.__validate_latter__["constraint"] = []
        else:
            self.__validate_latter__["unique"].clear()
            self.__validate_latter__["constraint"].clear()

    def _clear_globals(self):
        keys = list(self.__constraints__.keys())
        for k in keys:
            if k == "global":
                self.__constraints__["global"] = []
                continue
            del self.__constraints__[k]


class OpenApiObject(OpenApiBase, OpenApiValidator):
    """Base class for any /components/schemas object

    Every OpenApiObject is reuseable within the schema so it can
    exist in multiple locations within the hierarchy.
    That means it can exist in multiple locations as a
    leaf, parent/choice or parent.
    """

    __slots__ = ("__warnings__", "_properties", "_parent", "_choice")
    _DEFAULTS = {}
    _TYPES = {}
    _REQUIRED = []
    _STATUS = {}

    def __init__(self, parent=None, choice=None):
        super(OpenApiObject, self).__init__()
        self._parent = parent
        self._choice = choice
        self._properties = {}
        self.__warnings__ = []

    @property
    def parent(self):
        return self._parent

    def _set_choice(self, name):
        if self._has_choice(name):
            for enum in self._TYPES["choice"]["enum"]:
                if enum in self._properties and name != enum:
                    self._properties.pop(enum)
            self._properties["choice"] = name

    def _has_choice(self, name):
        if (
            "choice" in dir(self)
            and "_TYPES" in dir(self)
            and "choice" in self._TYPES
            and name in self._TYPES["choice"]["enum"]
        ):
            return True
        else:
            return False

    def _get_property(
        self, name, default_value=None, parent=None, choice=None
    ):
        if name in self._properties and self._properties[name] is not None:
            return self._properties[name]
        if isinstance(default_value, type) is True:
            self._set_choice(name)
            if "_choice" in default_value.__slots__:
                self._properties[name] = default_value(
                    parent=parent, choice=choice
                )
            else:
                self._properties[name] = default_value(parent=parent)
            if (
                "_DEFAULTS" in dir(self._properties[name])
                and "choice" in self._properties[name]._DEFAULTS
            ):
                getattr(
                    self._properties[name],
                    self._properties[name]._DEFAULTS["choice"],
                )
        else:
            if default_value is None and name in self._DEFAULTS:
                self._set_choice(name)
                self._properties[name] = self._DEFAULTS[name]
            else:
                self._properties[name] = default_value
        return self._properties[name]

    def _set_property(self, name, value, choice=None):
        if name == "choice":

            if (
                self.parent is None
                and value is not None
                and value not in self._TYPES["choice"]["enum"]
            ):
                raise Exception(
                    "%s is not a valid choice, valid choices are %s"
                    % (value, ", ".join(self._TYPES["choice"]["enum"]))
                )

            self._set_choice(value)
            if name in self._DEFAULTS and value is None:
                self._properties[name] = self._DEFAULTS[name]
        elif name in self._DEFAULTS and value is None:
            self._set_choice(name)
            self._properties[name] = self._DEFAULTS[name]
        else:
            self._set_choice(name)
            self._properties[name] = value
        # TODO: restore behavior
        # self._validate_unique_and_name(name, value)
        # self._validate_constraint(name, value)
        if (
            self._parent is not None
            and self._choice is not None
            and value is not None
        ):
            self._parent._set_property("choice", self._choice)

    def _encode(self):
        """Helper method for serialization"""
        output = {}
        self._raise_status_warnings(self, None)
        self._validate_required()
        for key, value in self._properties.items():
            self._validate_types(key, value)
            # TODO: restore behavior
            # self._validate_unique_and_name(key, value, True)
            # self._validate_constraint(key, value, True)
            if isinstance(value, (OpenApiObject, OpenApiIter)):
                output[key] = value._encode()
                if isinstance(value, OpenApiObject):
                    self._raise_status_warnings(key, value)
            elif value is not None:
                if (
                    self._TYPES.get(key, {}).get("format", "") == "int64"
                    or self._TYPES.get(key, {}).get("format", "") == "uint64"
                ):
                    value = str(value)
                elif (
                    self._TYPES.get(key, {}).get("itemformat", "") == "int64"
                    or self._TYPES.get(key, {}).get("itemformat", "")
                    == "uint64"
                ):
                    value = [str(v) for v in value]
                output[key] = value
                self._raise_status_warnings(key, value)
        return output

    def _decode(self, obj):
        dtypes = [list, str, int, float, bool]
        self._raise_status_warnings(self, None)
        for property_name, property_value in obj.items():
            if property_name in self._TYPES:
                ignore_warnings = False
                if isinstance(property_value, dict):
                    child = self._get_child_class(property_name)
                    if (
                        "choice" in child[1]._TYPES
                        and "_parent" in child[1].__slots__
                    ):
                        property_value = child[1](self, property_name)._decode(
                            property_value
                        )
                    elif "_parent" in child[1].__slots__:
                        property_value = child[1](self)._decode(property_value)
                    else:
                        property_value = child[1]()._decode(property_value)
                elif (
                    isinstance(property_value, list)
                    and property_name in self._TYPES
                    and self._TYPES[property_name]["type"] not in dtypes
                ):
                    child = self._get_child_class(property_name, True)
                    openapi_list = child[0]()
                    for item in property_value:
                        item = child[1]()._decode(item)
                        openapi_list._items.append(item)
                    property_value = openapi_list
                    ignore_warnings = True
                elif (
                    property_name in self._DEFAULTS and property_value is None
                ):
                    if isinstance(
                        self._DEFAULTS[property_name], tuple(dtypes)
                    ):
                        property_value = self._DEFAULTS[property_name]
                self._set_choice(property_name)
                # convert int64(will be string on wire) to to int
                if (
                    self._TYPES[property_name].get("format", "") == "int64"
                    or self._TYPES[property_name].get("format", "") == "uint64"
                ):
                    property_value = int(property_value)
                elif (
                    self._TYPES[property_name].get("itemformat", "") == "int64"
                    or self._TYPES[property_name].get("itemformat", "")
                    == "uint64"
                ):
                    property_value = [int(v) for v in property_value]
                self._properties[property_name] = property_value
                # TODO: restore behavior
                # OpenApiStatus.warn(
                #     "{}.{}".format(type(self).__name__, property_name), self
                # )
                if not ignore_warnings:
                    self._raise_status_warnings(property_name, property_value)
            self._validate_types(property_name, property_value)
            # TODO: restore behavior
            # self._validate_unique_and_name(property_name, property_value, True)
            # self._validate_constraint(property_name, property_value, True)
        self._validate_required()
        return self

    def _get_child_class(self, property_name, is_property_list=False):
        list_class = None
        class_name = self._TYPES[property_name]["type"]
        module = globals().get(self.__module__)
        if module is None:
            module = importlib.import_module(self.__module__)
            globals()[self.__module__] = module
        object_class = getattr(module, class_name)
        if is_property_list is True:
            list_class = object_class
            object_class = getattr(module, class_name[0:-4])
        return (list_class, object_class)

    def __str__(self):
        return self.serialize(encoding=self.YAML)

    def __deepcopy__(self, memo):
        """Creates a deep copy of the current object"""
        return self.__class__().deserialize(self.serialize())

    def __copy__(self):
        """Creates a deep copy of the current object"""
        return self.__deepcopy__(None)

    def __eq__(self, other):
        return self.__str__() == other.__str__()

    def clone(self):
        """Creates a deep copy of the current object"""
        return self.__deepcopy__(None)

    def _validate_required(self):
        """Validates the required properties are set
        Use getattr as it will set any defaults prior to validating
        """
        if getattr(self, "_REQUIRED", None) is None:
            return
        for name in self._REQUIRED:
            if self._properties.get(name) is None:
                msg = (
                    "{} is a mandatory property of {}"
                    " and should not be set to None".format(
                        name,
                        self.__class__,
                    )
                )
                raise ValueError(msg)

    def _validate_types(self, property_name, property_value):
        common_data_types = [list, str, int, float, bool]
        if property_name not in self._TYPES:
            # raise ValueError("Invalid Property {}".format(property_name))
            return
        details = self._TYPES[property_name]
        if (
            property_value is None
            and property_name not in self._DEFAULTS
            and property_name not in self._REQUIRED
        ):
            return
        if "enum" in details and property_value not in details["enum"]:
            msg = (
                "property {} shall be one of these"
                " {} enum, but got {} at {}"
            )
            raise TypeError(
                msg.format(
                    property_name,
                    details["enum"],
                    property_value,
                    self.__class__,
                )
            )
        if details["type"] in common_data_types and "format" not in details:
            msg = "property {} shall be of type {} at {}".format(
                property_name, details["type"], self.__class__
            )
            self.types_validation(
                property_value,
                details["type"],
                msg,
                details.get("itemtype"),
                details.get("minimum"),
                details.get("maximum"),
                details.get("minLength"),
                details.get("maxLength"),
            )

        if details["type"] not in common_data_types:
            class_name = details["type"]
            # TODO Need to revisit importlib
            module = importlib.import_module(self.__module__)
            object_class = getattr(module, class_name)
            if not isinstance(property_value, object_class):
                msg = "property {} shall be of type {}," " but got {} at {}"
                raise TypeError(
                    msg.format(
                        property_name,
                        class_name,
                        type(property_value),
                        self.__class__,
                    )
                )
        if "format" in details:
            msg = "Invalid {} format, expected {} at {}".format(
                property_value, details["format"], self.__class__
            )
            _type = (
                details["type"]
                if details["type"] is list
                else details["format"]
            )
            self.types_validation(
                property_value,
                _type,
                msg,
                details["format"],
                details.get("minimum"),
                details.get("maximum"),
                details.get("minLength"),
                details.get("maxLength"),
            )

    def validate(self):
        self._validate_required()
        for key, value in self._properties.items():
            self._validate_types(key, value)
        # TODO: restore behavior
        # self._validate_coded()

    def get(self, name, with_default=False):
        """
        getattr for openapi object
        """
        if self._properties.get(name) is not None:
            return self._properties[name]
        elif with_default:
            # TODO need to find a way to avoid getattr
            choice = (
                self._properties.get("choice")
                if "choice" in dir(self)
                else None
            )
            getattr(self, name)
            if "choice" in dir(self):
                if choice is None and "choice" in self._properties:
                    self._properties.pop("choice")
                else:
                    self._properties["choice"] = choice
            return self._properties.pop(name)
        return None

    def _raise_status_warnings(self, property_name, property_value):
        if len(self._STATUS) > 0:

            if isinstance(property_name, OpenApiObject):
                if "self" in self._STATUS and property_value is None:
                    print("[WARNING]: %s" % self._STATUS["self"])

                return

            enum_key = "%s.%s" % (property_name, property_value)
            if property_name in self._STATUS:
                print("[WARNING]: %s" % self._STATUS[property_name])
            elif enum_key in self._STATUS:
                print("[WARNING]: %s" % self._STATUS[enum_key])


class OpenApiIter(OpenApiBase):
    """Container class for OpenApiObject

    Inheriting classes contain 0..n instances of an OpenAPI components/schemas
    object.
    - config.flows.flow(name="1").flow(name="2").flow(name="3")

    The __getitem__ method allows getting an instance using ordinal.
    - config.flows[0]
    - config.flows[1:]
    - config.flows[0:1]
    - f1, f2, f3 = config.flows

    The __iter__ method allows for iterating across the encapsulated contents
    - for flow in config.flows:
    """

    __slots__ = ("_index", "_items")
    _GETITEM_RETURNS_CHOICE_OBJECT = False

    def __init__(self):
        super(OpenApiIter, self).__init__()
        self._index = -1
        self._items = []

    def __len__(self):
        return len(self._items)

    def _getitem(self, key):
        found = None
        if isinstance(key, int):
            found = self._items[key]
        elif isinstance(key, slice) is True:
            start, stop, step = key.indices(len(self))
            sliced = self.__class__()
            for i in range(start, stop, step):
                sliced._items.append(self._items[i])
            return sliced
        elif isinstance(key, str):
            for item in self._items:
                if item.name == key:
                    found = item
        if found is None:
            raise IndexError()
        if (
            self._GETITEM_RETURNS_CHOICE_OBJECT is True
            and found._properties.get("choice") is not None
            and found._properties.get(found._properties["choice"]) is not None
        ):
            return found._properties[found._properties["choice"]]
        return found

    def _iter(self):
        self._index = -1
        return self

    def _next(self):
        if self._index + 1 >= len(self._items):
            raise StopIteration
        else:
            self._index += 1
        return self.__getitem__(self._index)

    def __getitem__(self, key):
        raise NotImplementedError("This should be overridden by the generator")

    def _add(self, item):
        self._items.append(item)
        self._index = len(self._items) - 1

    def remove(self, index):
        del self._items[index]
        self._index = len(self._items) - 1

    def append(self, item):
        """Append an item to the end of OpenApiIter
        TBD: type check, raise error on mismatch
        """
        self._instanceOf(item)
        self._add(item)
        return self

    def clear(self):
        del self._items[:]
        self._index = -1

    def set(self, index, item):
        self._instanceOf(item)
        self._items[index] = item
        return self

    def _encode(self):
        return [item._encode() for item in self._items]

    def _decode(self, encoded_list):
        item_class_name = self.__class__.__name__.replace("Iter", "")
        module = importlib.import_module(self.__module__)
        object_class = getattr(module, item_class_name)
        self.clear()
        for item in encoded_list:
            self._add(object_class()._decode(item))

    def __copy__(self):
        raise NotImplementedError(
            "Shallow copy of OpenApiIter objects is not supported"
        )

    def __deepcopy__(self, memo):
        raise NotImplementedError(
            "Deep copy of OpenApiIter objects is not supported"
        )

    def __str__(self):
        return yaml.safe_dump(self._encode())

    def __eq__(self, other):
        return self.__str__() == other.__str__()

    def _instanceOf(self, item):
        raise NotImplementedError(
            "validating an OpenApiIter object is not supported"
        )



"""
To build distribution: python setup.py sdist --formats=gztar bdist_wheel --universal
"""
import os
import setuptools

pkg_name = "openapiart"
version = "0.2.15"

base_dir = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(base_dir, "README.md")) as fid:
    long_description = fid.read()

requirements_path = os.path.join(base_dir, "openapiart", "requirements.txt")
test_req_path = os.path.join(base_dir, "test_requirements.txt")
installation_requires = []
test_requires = []
if os.path.exists(requirements_path) is False:
    raise Exception("Could not find requirements path")
with open(requirements_path) as f:
    installation_requires = f.read().splitlines()
    if "--prefer-binary" in installation_requires:
        installation_requires.remove("--prefer-binary")
if os.path.exists(test_req_path):
    with open("test_requirements.txt") as f:
        test_requires = f.read().splitlines()

setuptools.setup(
    name=pkg_name,
    version=version,
    description="The OpenAPI Artifact Generator Python Package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/open-traffic-generator/openapiart",
    author="https://github.com/open-traffic-generator/openapiart",
    author_email="andy.balogh@keysight.com",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing :: Traffic Generation",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
    keywords="testing openapi artifact generator",
    package_data={"openapiart": ["*.go", "goserver/*.go", "*.txt"]},
    include_package_data=True,
    packages=[pkg_name],
    python_requires=">=2.7, <4",
    install_requires=installation_requires,
    extras_require={"testing": test_requires},
    test_suite="tests",
)



import sys
import os

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", "art"))
import sanity
import time


def test_perf(api):
    start = time.time()
    config = sanity.Api().prefix_config()
    config.a = "abc"
    config.b = 0.2
    config.c = 20
    config.required_object.e_a = 10.2
    config.required_object.e_b = 33.1
    config.response = config.STATUS_200
    config.optional_object.e_a = 11.2
    config.optional_object.e_b = 21.1
    config.d_values = [config.A]
    config.e.e_a = 1.1
    config.e.e_b = 2.1
    config.f.f_a = "FA"
    config.g.add()
    config.h = False
    config.i = "1010100"
    j = config.j.add()
    j.j_a.e_a = 10.1
    j.j_a.e_b = 2.1
    j.j_b.f_b = 3.1
    config.k.e_object.e_a = 1.0
    config.k.e_object.e_b = 0.1
    config.k.f_object.f_a
    config.l.double = 1.1
    config.l.float = 2.1
    config.l.hex = "0x123"
    config.l.integer = 23
    config.l.ipv4 = "10.1.1.1"
    config.l.ipv6 = "::"
    config.l.mac = "aa:bb:cc:11:22:33"
    config.l.string_param = "abcd"
    config.ieee_802_1qbb = True
    config.space_1 = 10
    config.full_duplex_100_mb = 20
    config.mandatory.required_param = "required"
    config.list_of_string_values = ["abc", "efg"]
    config.list_of_integer_values = [1, 2, 3, 4]
    config.level.l1_p1.l2_p1.l3_p1 = "L3P1"
    config.ipv4_pattern.ipv4.value = "10.1.1.1"
    config.ipv6_pattern.ipv6.values = ["::", "::1"]
    config.mac_pattern.mac.increment.start = "aa:aa:bb:bb:cc:cc"
    config.mac_pattern.mac.increment.step = "00:01:00:00:10:00"
    config.mac_pattern.mac.increment.count = 10
    config.integer_pattern.integer.decrement.start = 200
    config.integer_pattern.integer.decrement.step = 2
    config.integer_pattern.integer.decrement.count = 100
    config.checksum_pattern.checksum.generated = (
        config.checksum_pattern.checksum.GOOD
    )
    config.validate()
    end = time.time()
    print("Time taken for the manual config %f ms" % ((end - start) * 1000))

    start = time.time()
    json = config.serialize()
    end = time.time()
    print("Time elapsed to serialize to Json %f ms" % ((end - start) * 1000))

    start = time.time()
    c1 = sanity.Api().prefix_config()
    c1.deserialize(json)
    end = time.time()
    print(
        "Time elapsed to deserialize from Json %f ms" % ((end - start) * 1000)
    )

    start = time.time()
    yaml = config.serialize(config.YAML)
    end = time.time()
    print("Time elapsed to serialize to yaml %f ms" % ((end - start) * 1000))

    start = time.time()
    c1 = sanity.Api().prefix_config()
    c1.deserialize(yaml)
    end = time.time()
    print(
        "Time elapsed to deserialize from yaml %f ms" % ((end - start) * 1000)
    )

    start = time.time()
    dt = config.serialize(config.DICT)
    end = time.time()
    print("Time elapsed to serialize to DICT %f ms" % ((end - start) * 1000))

    start = time.time()
    c1 = sanity.Api().prefix_config()
    c1.deserialize(dt)
    end = time.time()
    print(
        "Time elapsed to deserialize from DICT %f ms" % ((end - start) * 1000)
    )

    start = time.time()
    api.set_config(config)
    end = time.time()
    print(
        "Time elapsed on set_config http call %f ms" % ((end - start) * 1000)
    )



import sys
import os
import importlib
import shutil
import yaml
import subprocess
import platform
from .requirements import generate_requirements


class OpenApiArt(object):
    """Bundle and generate artifacts from OpenAPI files.

    Args
    ----
    - api_files (list[str]): list of OpenAPI files that contain info and/or path
      keywords
    - protobuf_name (str): name of the .proto file that will be generated
    - artifact_dir (str): directory where artifacts will be created.
      Unless otherwise specified the default directory for generated artifacts
      is `current working directory/.art`.
    - extension_prefix (str): name of the python extension
    """

    def __init__(
        self,
        api_files,
        protobuf_name=None,
        artifact_dir=None,
        extension_prefix=None,
        proto_service=None,
        generate_version_api=False,
    ):
        self._output_dir = os.path.abspath(
            artifact_dir if artifact_dir is not None else "art"
        )
        self._doc_dir = os.path.abspath(
            # doc dir name is not getting ignored by git
            os.path.join(artifact_dir, "..", "proto_doc")
            if artifact_dir is not None
            else "doc"
        )
        self._go_sdk_package_dir = None
        self._protobuf_package_name = (
            protobuf_name if protobuf_name is not None else "sanity"
        )
        self._extension_prefix = (
            extension_prefix if extension_prefix is not None else "sanity"
        )
        self._proto_service = (
            proto_service if proto_service is not None else "Openapi"
        )

        print(
            "Artifact output directory: {output_dir}".format(
                output_dir=self._output_dir
            )
        )
        shutil.rmtree(self._output_dir, ignore_errors=True)
        self._api_files = api_files
        self._generate_version_api = generate_version_api
        self._bundle()
        self._get_info()
        self._get_license()
        self._document()

    def _get_license(self):
        license_name = self._bundler._content["info"]["license"]["name"]
        self._license = "License: {}".format(license_name)

    def _get_info(self):
        try:
            self._info = "{} {} \n{}".format(
                self._bundler._content["info"]["title"],
                self._bundler._content["info"]["version"],
                self._bundler._content["info"].get(
                    "description", "\nDescription not available"
                ),
            )
        except Exception as e:
            ex = Exception(
                "The following object and properties are REQUIRED: info, info.title, info.version [{}]".format(
                    e
                )
            )
            raise ex

    def _bundle(self):
        # bundle the yaml files
        module = importlib.import_module("openapiart.bundler")
        bundler_class = getattr(module, "Bundler")
        self._bundler = bundler_class(
            api_files=self._api_files,
            output_dir=self._output_dir,
            generate_version_api=self._generate_version_api,
        )
        self._bundler.bundle()
        self._api_version = self._bundler.get_api_version()
        # read the entire openapi file
        with open(self._bundler.openapi_filepath) as fp:
            self._openapi = yaml.safe_load(fp.read())

    def _document(self):
        """Try documenting the openapi using redoc-cli

        - Requires nodejs, npm, redoc-cli
        - npm install -g redoc-cli

        If the requirements are not installed/reachable then the document step
        will be bypassed but the generation will not fail.
        """
        try:
            process_args = [
                "redoc-cli",
                "bundle",
                self._bundler.openapi_filepath,
                "--output",
                os.path.join(self._output_dir, "openapi.html"),
            ]
            subprocess.check_call(
                process_args, shell=platform.system() == "Windows"
            )
        except Exception as e:
            print("Bypassed creation of static documentation: {}".format(e))

    def GenerateProtoDef(self, package_name):
        self._protobuf_package_name = package_name
        self._generate_proto_file()

    def GeneratePythonSdk(self, package_name, sdk_version=""):
        """Generates a Python UX Sdk
        Args
        ----
        - package_name (str): the name of the python module

        Example
        -------
        ```
        art = Openapiart(api_files=["<list of open_api_file_path>"], artifact_dir="./")
        art.GeneratePythonSdk(package_name="sanity")
        ```

        Output
        ------
        ```
        ./sanity
                |_ __init__.py
                |_ sanity.py
                |_ sanity_pb.py
                |_ sanity_grpc_pb.py
        ```
        """
        self._python_module_name = package_name
        self._python_sdk_version = sdk_version
        if not self.proto_file_exists():
            self._generate_proto_file()
        if self._python_module_name is not None:
            module = importlib.import_module("openapiart.generator")
            python_ux = getattr(module, "Generator")(
                self._bundler.openapi_filepath,
                self._python_module_name,
                self._protobuf_package_name,
                output_dir=self._output_dir,
                extension_prefix=self._extension_prefix,
                generate_version_api=self._generate_version_api,
                api_version=self._api_version,
                sdk_version=self._python_sdk_version,
            )
            python_ux.generate()
        try:
            python_sdk_dir = os.path.normpath(
                os.path.join(self._output_dir, self._python_module_name)
            )
            process_args = [
                sys.executable,
                "-m",
                "grpc_tools.protoc",
                "--python_out={}".format(python_sdk_dir),
                "--grpc_python_out={}".format(python_sdk_dir),
                "--proto_path={}".format(self._output_dir),
                "--experimental_allow_proto3_optional",
                "{}.proto".format(self._protobuf_package_name),
            ]
            print(
                "Generating python grpc stubs: {}".format(
                    " ".join(process_args)
                )
            )
            subprocess.check_call(process_args, shell=False)

            pb2_grpc_file = os.path.join(
                python_sdk_dir,
                "{}_pb2_grpc.py".format(self._protobuf_package_name),
            )
            current_text = (
                "import {proto_name}_pb2 as {proto_name}__pb2".format(
                    proto_name=self._protobuf_package_name
                )
            )
            new_text = "try:\n    {text}\nexcept ImportError:\n    from {pkg_name} {text}".format(
                text=current_text, pkg_name=self._python_module_name
            )
            with open(pb2_grpc_file) as f:
                file_contents = f.read().replace(current_text, new_text)
            with open(pb2_grpc_file, "w") as f:
                f.write(file_contents)
        except Exception as e:
            print("Bypassed creation of python stubs: {}".format(e))
        # Auto formatting generated python SDK with Black
        if sys.version_info[0] == 3:
            process_args = [
                "{} -m black".format(sys.executable),
                os.path.join(python_sdk_dir, self._python_module_name + ".py"),
            ]
            cmd = " ".join(process_args)
            print("Formatting Generated Python SDK: {}".format(cmd))
            subprocess.check_call(cmd, shell=True)
            generate_requirements(
                path=os.path.normpath(os.path.join(python_sdk_dir, ".."))
            )
        return self

    def GenerateGoSdk(self, package_dir, package_name, sdk_version=""):
        """Generates a Go UX Sdk

        Args
        ----
        - package_dir: Go mod package dir published under go.mod
        - package_name: Name of the Go package to generate

        Example
        -------
        ```
        art = Openapiart(api_files=["list of open_api_file_path"], artifact_dir="./")
        art.GenerateGoSdk(
            package_dir="github.com/<path to repo>/$package_name",
            package_name="sanity"
        )
        ```

        Output
        ------
        ```
        ./sanity
                |_ sanity.go
                |_ go.mod
                |_ go.sum
                |_ sanitypb
                            |_ sanitypb.pb.go
                            |_ sanitypb_grpc.go
        ```
        """

        self._go_sdk_package_dir = package_dir
        self._go_sdk_package_name = package_name
        self._go_sdk_version = sdk_version
        if not self.proto_file_exists():
            self._generate_proto_file()
        if self._go_sdk_package_dir and self._protobuf_package_name:
            go_sdk_output_dir = os.path.normpath(
                os.path.join(
                    self._output_dir,
                    "..",
                    os.path.split(self._go_sdk_package_dir)[-1],
                )
            )
            go_protobuffer_out_dir = os.path.normpath(
                os.path.join(go_sdk_output_dir, self._protobuf_package_name)
            )
            if not os.path.exists(go_protobuffer_out_dir):
                os.makedirs(go_protobuffer_out_dir)
            process_args = [
                "protoc",
                "--go_opt=paths=source_relative",
                "--go-grpc_opt=paths=source_relative",
                "--go_out={}".format(go_protobuffer_out_dir),
                "--go-grpc_out={}".format(go_protobuffer_out_dir),
                "--proto_path={}".format(self._output_dir),
                "--experimental_allow_proto3_optional",
                "{}.proto".format(self._protobuf_package_name),
            ]
            cmd = " ".join(process_args)
            print("Generating go gRPC stubs: {}".format(cmd))
            subprocess.check_call(cmd, shell=True)

        # this generates the go ux module
        if self._protobuf_package_name and self._go_sdk_package_dir:
            module = importlib.import_module("openapiart.openapiartgo")
            go_ux = getattr(module, "OpenApiArtGo")(
                **{
                    "info": self._info,
                    "license": self._license,
                    "protobuf_package_name": self._protobuf_package_name,
                    "go_sdk_package_dir": self._go_sdk_package_dir,
                    "go_sdk_package_name": self._go_sdk_package_name,
                    "output_dir": self._output_dir,
                    "proto_service": self._proto_service,
                    "generate_version_api": self._generate_version_api,
                    "api_version": self._api_version,
                    "sdk_version": self._go_sdk_version,
                }
            )
            print("Generating go ux sdk: {}".format(" ".join(process_args)))
            go_ux.generate(self._openapi)
        return self

    def GenerateGoServer(self, module_path, models_prefix="", models_path=""):
        outputfolder = module_path.split("/")[-1]
        go_server_output_dir = os.path.normpath(
            os.path.join(self._output_dir, "..", outputfolder)
        )
        module = importlib.import_module("openapiart.goserver.goserver")
        servergen = getattr(module, "GoServerGenerator")(
            **{
                "openapi": self._openapi,
                "output_root_path": go_server_output_dir,
                "module_path": module_path,
                "models_prefix": models_prefix,
                "models_path": models_path,
            }
        )
        servergen.generate()
        return self

    def GoTidy(self, relative_package_dir):
        go_server_output_dir = os.path.normpath(
            os.path.join(self._output_dir, "..", relative_package_dir)
        )
        module = importlib.import_module("openapiart.gotidy")
        tidy = getattr(module, "GoTidy")(
            **{
                "output_root_path": go_server_output_dir,
            }
        )
        tidy.goTidy()
        return self

    def _generate_proto_file(self):
        if self._protobuf_package_name is None:
            self._protobuf_package_name = "default"
        module = importlib.import_module("openapiart.openapiartprotobuf")
        protobuf = getattr(module, "OpenApiArtProtobuf")(
            **{
                "info": self._info,
                "license": self._license,
                "protobuf_package_name": self._protobuf_package_name,
                "go_sdk_package_dir": self._go_sdk_package_dir,
                "output_dir": self._output_dir,
                "proto_service": self._proto_service,
                "doc_dir": self._doc_dir,
            }
        )
        protobuf.generate(self._openapi)

    def proto_file_exists(self):
        proto_file_path = os.path.normpath(
            os.path.join(
                self._output_dir,
                "{}.proto".format(self._protobuf_package_name),
            )
        )
        return os.path.exists(proto_file_path)

    @property
    def output_dir(self):
        return self._output_dir

    @property
    def python_module_name(self):
        return self._python_module_name



import importlib

module = importlib.import_module("sanity")


def test_int64(default_config):
    value1 = default_config._TYPES.get("full_duplex_100_mb")
    value2 = default_config._TYPES.get("integer64_list")
    assert value1.get("format") is not None
    assert value2.get("itemformat") is not None
    assert value1.get("format") == "int64"
    assert value2.get("itemformat") == "int64"
    default_config.full_duplex_100_mb = 100
    default_config.integer64_list = [2000]
    data = default_config.serialize("dict")
    assert isinstance(data["full_duplex_100_mb"], str)
    assert isinstance(data["integer64_list"][0], str)

    config = module.Api().prefix_config()
    config.deserialize(data)
    assert isinstance(config.full_duplex_100_mb, int)
    assert isinstance(config.integer64_list[0], int)



import pytest


def test_formats_sanity(config):
    config.l.string_param = "asdf"
    config.l.integer = 88
    config.l.float = 22.3
    config.l.double = 2342.222
    config.l.mac = "00:00:fa:ce:fa:ce"
    config.l.ipv4 = "1.1.1.1"
    config.l.ipv6 = "::02"
    config.l.hex = "0102030405060708090a0b0c0d0e0f"
    yaml = config.serialize(encoding=config.YAML)
    config.deserialize(yaml)
    print(yaml)


@pytest.mark.parametrize("value", [33.4, 100])
def test_formats_bad_string(config, value):
    config.l.string_param = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail(
            "Value {value} was successfully validated".format(value=value)
        )
    except TypeError:
        pass


@pytest.mark.parametrize("value", [33.4, "asdf"])
def test_formats_bad_integer(config, value):
    config.l.integer = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize("value", [6, 100, -20])
def test_formats_integer_to_be_removed(config, value):
    try:
        config.l.integer = value
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize("value", ["1.1.1.1", "01.002.003.4", "0.0.0.0"])
def test_formats_good_ipv4(config, value):
    config.l.ipv4 = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
    except TypeError:
        pytest.fail("Value {} was not valid".format(value))


@pytest.mark.parametrize(
    "value",
    [
        "1.1. 1.1",
        33.4,
        "asdf",
        100,
        -20,
        "::01",
        "1.1.1.1.1",
        "256.256.256.256",
        "-255.-255.-255.-255",
    ],
)
def test_formats_bad_ipv4(config, value):
    config.l.ipv4 = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize("value", ["1.1", "1.1.1", " 1.1.1.1 "])
def test_formats_ipv4_to_be_removed(config, value):
    try:
        config.l.ipv4 = value
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize(
    "value",
    [
        33.4,
        "asdf",
        "1.1.1.1",
        100,
        -20,
        "65535::65535",
        "ab: :ab",
        "ab:ab:ab",
        "ffff0::ffff0",
    ],
)
def test_formats_bad_ipv6(config, value):
    config.l.ipv6 = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize(
    "value",
    [
        1,
        2.2,
        "1.1.1.1",
        "::01",
        "00:00:00",
        "00:00:00:00:gg:00",
        "00:00:fa:ce:fa:ce:01",
        "255:255:255:255:255:255",
    ],
)
def test_formats_bad_mac(config, value):
    config.l.mac = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize(
    "value", [1, 2.2, "1.1.1.1", "::01", "00:00:fa:ce:fa:ce:01"]
)
def test_formats_bad_hex(config, value):
    config.l.hex = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


@pytest.mark.parametrize("value", ["1234567", "12"])
def test_string_length(config, value):
    config.str_len = value
    try:
        config.deserialize(config.serialize(encoding=config.YAML))
        pytest.fail("Value {} was successfully validated".format(value))
    except TypeError:
        pass


def test_int64_list(config, default_config):
    config.integer64 = 10
    config.integer64_list = [1, 2]
    data = config.serialize("dict")
    assert isinstance(data["integer64"], str)
    for d in data["integer64_list"]:
        assert isinstance(d, str)
    default_config.deserialize(data)
    assert isinstance(default_config.integer64, int)
    for d in default_config.integer64_list:
        assert isinstance(d, int)


if __name__ == "__main__":
    pytest.main(["-v", "-s", __file__])
